﻿using System;
using System.Collections.Generic;
using System.Reflection;
using System.Data;
using System.Data.Common;

namespace Tsanie.Web.Models.Utils {

    /// <summary>
    /// 数据库连接帮助类
    /// </summary>
    public class DBHelper {

        #region - 静态 -
        private readonly static string _connStr;
        private readonly static DbProviderFactory _provider;

        /// <summary>
        /// 静态构造
        /// </summary>
        static DBHelper() {
            System.Collections.Specialized.NameValueCollection appSettings
                = System.Configuration.ConfigurationManager.AppSettings;
            // 连接字符串
            _connStr = appSettings["connStr"];
            // 优先provider，如果存在则用DbProviderFactories.GetFactory()创建
            string provider = appSettings["provider"];
            if (provider != null) {
                _provider = DbProviderFactories.GetFactory(provider);
                return;
            }
            // 否则使用Assembly.LoadFrom()载入程序集
            // 因为DbProviderFactories.GetFactory()无法创建MySqlClientFactory
            string factory = appSettings["factory"];
            Type typeProvider = Assembly
                .LoadFrom(AppDomain.CurrentDomain.BaseDirectory + appSettings["driver"])
                .GetType(factory);
            _provider = typeProvider
                .GetConstructor(Type.EmptyTypes)
                .Invoke(null) as DbProviderFactory;
        }
        #endregion

        #region - 属性 -
        /// <summary>
        /// 获取或设置SQL语句文本
        /// </summary>
        public string Command { get; set; }
        /// <summary>
        /// 获取或设置语句类型
        /// </summary>
        public CommandType CommandType { get; set; }
        /// <summary>
        /// 获取语句参数
        /// </summary>
        public List<DbParameter> Parameters { get; private set; }
        #endregion

        #region - 构造 -
        private DbConnection _conn;

        /// <summary>
        /// 构造数据库助手
        /// </summary>
        public DBHelper() : this(null, CommandType.Text) { }

        /// <summary>
        /// 构造数据库助手
        /// </summary>
        /// <param name="sql">要执行的SQL语句</param>
        public DBHelper(string sql) : this(sql, CommandType.Text) { }

        /// <summary>
        /// 构造数据库助手
        /// </summary>
        /// <param name="sql">要执行的SQL语句</param>
        /// <param name="type">语句类型</param>
        public DBHelper(string sql, CommandType type) {
            _conn = _provider.CreateConnection();
            _conn.ConnectionString = _connStr;
            Command = sql;
            CommandType = type;
            Parameters = null;
        }
        #endregion

        #region - 公共方法 -
        /// <summary>
        /// 添加参数
        /// </summary>
        /// <param name="parameterName">参数名</param>
        /// <param name="value">参数值</param>
        /// <returns>参数</returns>
        public DbParameter AddParameter(string parameterName, object value) {
            DbParameter param = _provider.CreateParameter();
            param.ParameterName = parameterName;
            param.Value = value;
            return AddParameter(param);
        }

        /// <summary>
        /// 添加参数
        /// </summary>
        /// <param name="parameterName">参数名</param>
        /// <param name="value">参数值</param>
        /// <param name="dbType">参数数据类型</param>
        /// <returns>参数</returns>
        public DbParameter AddParameter(string parameterName, object value, DbType dbType) {
            DbParameter param = _provider.CreateParameter();
            param.ParameterName = parameterName;
            param.Value = value;
            param.DbType = dbType;
            return AddParameter(param);
        }

        /// <summary>
        /// 添加参数
        /// </summary>
        /// <param name="parameterName">参数名</param>
        /// <param name="value">参数值</param>
        /// <param name="dbType">参数数据类型</param>
        /// <param name="size">参数长度</param>
        /// <returns>参数</returns>
        public DbParameter AddParameter(string parameterName, object value, DbType dbType, int size) {
            return AddParameter(parameterName, value, dbType, size, ParameterDirection.Input);
        }

        /// <summary>
        /// 添加参数
        /// </summary>
        /// <param name="parameterName">参数名</param>
        /// <param name="value">参数值</param>
        /// <param name="dbType">参数数据类型</param>
        /// <param name="size">参数长度</param>
        /// <param name="direction">参数类型</param>
        /// <returns>参数</returns>
        public DbParameter AddParameter(string parameterName, object value, DbType dbType,
            int size, ParameterDirection direction) {
            DbParameter param = _provider.CreateParameter();
            param.ParameterName = parameterName;
            param.Value = value;
            param.DbType = dbType;
            param.Size = size;
            param.Direction = direction;
            return AddParameter(param);
        }

        /// <summary>
        /// 执行非查询的SQL语句
        /// </summary>
        /// <returns>DAO处理结果</returns>
        public int ExecuteSql() {
            using (DbCommand cmd = PrepareCommand()) {
                try {
                    int rows = cmd.ExecuteNonQuery();
                    cmd.Parameters.Clear();
                    return rows;
                } catch (DbException e) {
                    throw e;
                } finally {
                    CloseConnection();
                }
            }
        }

        /// <summary>
        /// 执行查询SQL语句
        /// </summary>
        /// <returns>查询列表</returns>
        public IEnumerable<TEntity> ExecuteQuery<TEntity>() where TEntity : new() {
            using (DbCommand cmd = PrepareCommand()) {
                DbDataReader reader = null;
                try {
                    reader = cmd.ExecuteReader();
                    return createEnumerable<TEntity>(reader);
                } catch (DbException e) {
                    throw e;
                } finally {
                    CloseConnection(reader);
                }
            }
        }

        /// <summary>
        /// 执行查询SQL语句
        /// </summary>
        /// <returns>查询到的单个对象</returns>
        public TEntity ExecuteQuerySingle<TEntity>() where TEntity : new() {
            using (DbCommand cmd = PrepareCommand()) {
                DbDataReader reader = null;
                try {
                    reader = cmd.ExecuteReader();
                    if (reader.Read()) {
                        return createSingle<TEntity>(reader);
                    }
                    return default(TEntity);
                } catch (DbException e) {
                    throw e;
                } finally {
                    CloseConnection(reader);
                }
            }
        }

        /// <summary>
        /// 执行查询SQL语句
        /// </summary>
        /// <returns>查询结果表</returns>
        public DataTable ExecuteQueryTable() {
            using (DbCommand cmd = PrepareCommand()) {
                DbDataAdapter adapter = _provider.CreateDataAdapter();
                adapter.SelectCommand = cmd;
                try {
                    DataTable table = new DataTable();
                    adapter.Fill(table);
                    return table;
                } catch (DbException ex) {
                    throw ex;
                } finally {
                    CloseConnection();
                }
            }
        }

        /// <summary>
        /// 执行查询SQL语句并返回个数
        /// </summary>
        /// <returns>查询到的个数</returns>
        public long ExecuteQueryCount() {
            using (DbCommand cmd = PrepareCommand()) {
                try {
                    object result = cmd.ExecuteScalar();
                    return result == null ? -1 : (long)result;
                } catch (DbException e) {
                    throw e;
                } finally {
                    CloseConnection();
                }
            }
        }
        #endregion

        #region - 私有方法 -
        /// <summary>
        /// 向参数集合里添加参数
        /// </summary>
        /// <param name="param"></param>
        private DbParameter AddParameter(DbParameter param) {
            if (Parameters == null)
                Parameters = new List<DbParameter>();
            Parameters.Add(param);
            return param;
        }
        /// <summary>
        /// 准备SQL命令对象
        /// </summary>
        /// <returns>SQL命令对象</returns>
        private DbCommand PrepareCommand() {
            try {
                if (_conn.State != ConnectionState.Open)
                    _conn.Open();
                DbCommand cmd = _conn.CreateCommand();
                cmd.CommandText = Command;
                cmd.CommandType = CommandType;
                if (Parameters != null && Parameters.Count > 0) {
                    foreach (DbParameter param in Parameters) {
                        cmd.Parameters.Add(param);
                    }
                }
                return cmd;
            } catch (DbException e) {
                throw e;
            }
        }

        /// <summary>
        /// 创建集合
        /// </summary>
        /// <typeparam name="TEntity">实体类型</typeparam>
        /// <param name="reader">数据读取器</param>
        /// <returns>实体集合</returns>
        private IEnumerable<TEntity> createEnumerable<TEntity>(DbDataReader reader) where TEntity : new() {
            List<TEntity> list = new List<TEntity>();
            // 遍历结果集
            while (reader.Read()) {
                // 将新添加的实例存放在ArrayList中
                list.Add(createSingle<TEntity>(reader));
            }
            return list;
        }

        /// <summary>
        /// 创建实体
        /// </summary>
        /// <typeparam name="TEntity">实体类型</typeparam>
        /// <param name="reader">数据读取器</param>
        /// <returns>实体对象</returns>
        private TEntity createSingle<TEntity>(DbDataReader reader) where TEntity : new() {
            // 创建该类的实例
            TEntity curObj = new TEntity();
            Type type = curObj.GetType();

            // 遍历所有的公有属性
            foreach (PropertyInfo pInfo in type.GetProperties()) {
                MethodInfo method = pInfo.GetSetMethod(false);
                if (method != null) {
                    try {
                        object value = reader[pInfo.Name];
                        if (value == null || value is System.DBNull) {
                            method.Invoke(curObj, new object[] { null });
                        } else {
                            method.Invoke(curObj, new object[] { reader[pInfo.Name] });
                        }
                    } catch (Exception e) {
                        throw e;
                    }
                }
            }
            return curObj;
        }

        /// <summary>
        /// 关闭连接
        /// </summary>
        private void CloseConnection() {
            CloseConnection(null);
        }

        /// <summary>
        /// 关闭连接
        /// </summary>
        private void CloseConnection(DbDataReader reader) {
            if (reader != null && !reader.IsClosed) {
                reader.Close();
            }
            if (_conn != null && _conn.State == ConnectionState.Open) {
                _conn.Close();
            }
        }
        #endregion

    }
}
